@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -26,7 +26,7 @@ Module attributes:
 
 import os
 import operator
-import typing  # pylint: disable=unused-import,useless-suppression
+import typing
 
 from PyQt5.QtGui import QFont
 from PyQt5.QtWebEngineWidgets import (QWebEngineSettings, QWebEngineProfile,
@@ -35,16 +35,17 @@ from PyQt5.QtWebEngineWidgets import (QWebEngineSettings, QWebEngineProfile,
 from qutebrowser.browser.webengine import spell, webenginequtescheme
 from qutebrowser.config import config, websettings
 from qutebrowser.config.websettings import AttributeInfo as Attr
-from qutebrowser.utils import utils, standarddir, qtutils, message, log
+from qutebrowser.utils import (utils, standarddir, qtutils, message, log,
+                               urlmatch, usertypes)
 
 # The default QWebEngineProfile
-default_profile = None  # type: typing.Optional[QWebEngineProfile]
+default_profile = typing.cast(QWebEngineProfile, None)
 # The QWebEngineProfile used for private (off-the-record) windows
 private_profile = None  # type: typing.Optional[QWebEngineProfile]
 # The global WebEngineSettings object
-global_settings = None
+global_settings = typing.cast('WebEngineSettings', None)
 
-default_user_agent = None
+parsed_user_agent = None
 
 
 class _SettingsWrapper:
@@ -59,33 +60,40 @@ class _SettingsWrapper:
         if private_profile:
             self._settings.append(private_profile.settings())
 
-    def setAttribute(self, *args, **kwargs):
+    def setAttribute(self, attribute, on):
         for settings in self._settings:
-            settings.setAttribute(*args, **kwargs)
+            settings.setAttribute(attribute, on)
 
-    def setFontFamily(self, *args, **kwargs):
+    def setFontFamily(self, which, family):
         for settings in self._settings:
-            settings.setFontFamily(*args, **kwargs)
+            settings.setFontFamily(which, family)
 
-    def setFontSize(self, *args, **kwargs):
+    def setFontSize(self, fonttype, size):
         for settings in self._settings:
-            settings.setFontSize(*args, **kwargs)
+            settings.setFontSize(fonttype, size)
 
-    def setDefaultTextEncoding(self, *args, **kwargs):
+    def setDefaultTextEncoding(self, encoding):
         for settings in self._settings:
-            settings.setDefaultTextEncoding(*args, **kwargs)
+            settings.setDefaultTextEncoding(encoding)
 
-    def testAttribute(self, *args, **kwargs):
-        return self._settings[0].testAttribute(*args, **kwargs)
+    def setUnknownUrlSchemePolicy(self, policy):
+        for settings in self._settings:
+            settings.setUnknownUrlSchemePolicy(policy)
 
-    def fontSize(self, *args, **kwargs):
-        return self._settings[0].fontSize(*args, **kwargs)
+    def testAttribute(self, attribute):
+        return self._settings[0].testAttribute(attribute)
 
-    def fontFamily(self, *args, **kwargs):
-        return self._settings[0].fontFamily(*args, **kwargs)
+    def fontSize(self, fonttype):
+        return self._settings[0].fontSize(fonttype)
 
-    def defaultTextEncoding(self, *args, **kwargs):
-        return self._settings[0].defaultTextEncoding(*args, **kwargs)
+    def fontFamily(self, which):
+        return self._settings[0].fontFamily(which)
+
+    def defaultTextEncoding(self):
+        return self._settings[0].defaultTextEncoding()
+
+    def unknownUrlSchemePolicy(self):
+        return self._settings[0].unknownUrlSchemePolicy()
 
 
 class WebEngineSettings(websettings.AbstractSettings):
@@ -150,6 +158,19 @@ class WebEngineSettings(websettings.AbstractSettings):
         'fonts.web.family.fantasy': QWebEngineSettings.FantasyFont,
     }
 
+    # Only Qt >= 5.11 support UnknownUrlSchemePolicy
+    try:
+        _UNKNOWN_URL_SCHEME_POLICY = {
+            'disallow':
+                QWebEngineSettings.DisallowUnknownUrlSchemes,
+            'allow-from-user-interaction':
+                QWebEngineSettings.AllowUnknownUrlSchemesFromUserInteraction,
+            'allow-all':
+                QWebEngineSettings.AllowAllUnknownUrlSchemes,
+        }
+    except AttributeError:
+        _UNKNOWN_URL_SCHEME_POLICY = None
+
     # Mapping from WebEngineSettings::initDefaults in
     # qtwebengine/src/core/web_engine_settings.cpp
     _FONT_TO_QFONT = {
@@ -161,6 +182,33 @@ class WebEngineSettings(websettings.AbstractSettings):
         QWebEngineSettings.FantasyFont: QFont.Fantasy,
     }
 
+    def set_unknown_url_scheme_policy(
+            self, policy: typing.Union[str, usertypes.Unset]) -> bool:
+        """Set the UnknownUrlSchemePolicy to use.
+
+        Return:
+            True if there was a change, False otherwise.
+        """
+        old_value = self._settings.unknownUrlSchemePolicy()
+        if isinstance(policy, usertypes.Unset):
+            self._settings.resetUnknownUrlSchemePolicy()
+            new_value = self._settings.unknownUrlSchemePolicy()
+        else:
+            new_value = self._UNKNOWN_URL_SCHEME_POLICY[policy]
+            self._settings.setUnknownUrlSchemePolicy(new_value)
+        return old_value != new_value
+
+    def _update_setting(self, setting, value):
+        if setting == 'content.unknown_url_scheme_policy':
+            if self._UNKNOWN_URL_SCHEME_POLICY:
+                return self.set_unknown_url_scheme_policy(value)
+            return False
+        return super()._update_setting(setting, value)
+
+    def init_settings(self):
+        super().init_settings()
+        self.update_setting('content.unknown_url_scheme_policy')
+
     def __init__(self, settings):
         super().__init__(settings)
         # Attributes which don't exist in all Qt versions.
@@ -228,7 +276,9 @@ class ProfileSetter:
         per-domain values), but this one still gets used for things like
         window.navigator.userAgent/.languages in JS.
         """
-        self._profile.setHttpUserAgent(config.val.content.headers.user_agent)
+        user_agent = websettings.user_agent()
+        self._profile.setHttpUserAgent(user_agent)
+
         accept_language = config.val.content.headers.accept_language
         if accept_language is not None:
             self._profile.setHttpAcceptLanguage(accept_language)
@@ -296,12 +346,22 @@ def _update_settings(option):
             private_profile.setter.set_dictionary_language(warn=False)
 
 
+def _init_user_agent_str(ua):
+    global parsed_user_agent
+    parsed_user_agent = websettings.UserAgent.parse(ua)
+
+
+def init_user_agent():
+    _init_user_agent_str(QWebEngineProfile.defaultProfile().httpUserAgent())
+
+
 def _init_profiles():
     """Init the two used QWebEngineProfiles."""
-    global default_profile, private_profile, default_user_agent
+    global default_profile, private_profile
 
     default_profile = QWebEngineProfile.defaultProfile()
-    default_user_agent = default_profile.httpUserAgent()
+    init_user_agent()
+
     default_profile.setter = ProfileSetter(default_profile)
     default_profile.setCachePath(
         os.path.join(standarddir.cache(), 'webengine'))
@@ -317,6 +377,50 @@ def _init_profiles():
         private_profile.setter.init_profile()
 
 
+def _init_site_specific_quirks():
+    if not config.val.content.site_specific_quirks:
+        return
+
+    # default_ua = ("Mozilla/5.0 ({os_info}) "
+    #               "AppleWebKit/{webkit_version} (KHTML, like Gecko) "
+    #               "{qt_key}/{qt_version} "
+    #               "{upstream_browser_key}/{upstream_browser_version} "
+    #               "Safari/{webkit_version}")
+    no_qtwe_ua = ("Mozilla/5.0 ({os_info}) "
+                  "AppleWebKit/{webkit_version} (KHTML, like Gecko) "
+                  "{upstream_browser_key}/{upstream_browser_version} "
+                  "Safari/{webkit_version}")
+    firefox_ua = "Mozilla/5.0 ({os_info}; rv:71.0) Gecko/20100101 Firefox/71.0"
+    new_chrome_ua = ("Mozilla/5.0 ({os_info}) "
+                     "AppleWebKit/537.36 (KHTML, like Gecko) "
+                     "Chrome/99 "
+                     "Safari/537.36")
+
+    user_agents = {
+        'https://web.whatsapp.com/': no_qtwe_ua,
+        'https://accounts.google.com/*': firefox_ua,
+        'https://*.slack.com/*': new_chrome_ua,
+        'https://docs.google.com/*': firefox_ua,
+    }
+
+    if not qtutils.version_check('5.9'):
+        user_agents['https://www.dell.com/support/*'] = new_chrome_ua
+
+    for pattern, ua in user_agents.items():
+        config.instance.set_obj('content.headers.user_agent', ua,
+                                pattern=urlmatch.UrlPattern(pattern),
+                                hide_userconfig=True)
+
+
+def _init_devtools_settings():
+    """Make sure the devtools always get images/JS permissions."""
+    for setting in ['content.javascript.enabled', 'content.images']:
+        for pattern in ['chrome-devtools://*', 'devtools://*']:
+            config.instance.set_obj(setting, True,
+                                    pattern=urlmatch.UrlPattern(pattern),
+                                    hide_userconfig=True)
+
+
 def init(args):
     """Initialize the global QWebSettings."""
     if (args.enable_webengine_inspector and
@@ -333,6 +437,9 @@ def init(args):
     global_settings = WebEngineSettings(_SettingsWrapper())
     global_settings.init_settings()
 
+    _init_site_specific_quirks()
+    _init_devtools_settings()
+
 
 def shutdown():
     pass
