@@ -24,6 +24,8 @@ import os.path
 import itertools
 import urllib
 import typing
+import glob
+import shutil
 
 from PyQt5.QtCore import QUrl, QObject, QPoint, QTimer, pyqtSlot
 from PyQt5.QtWidgets import QApplication
@@ -59,6 +61,16 @@ def init(parent=None):
         parent: The parent to use for the SessionManager.
     """
     base_path = os.path.join(standarddir.data(), 'sessions')
+
+    # WORKAROUND for https://github.com/qutebrowser/qutebrowser/issues/5359
+    backup_path = os.path.join(base_path, 'before-qt-515')
+    if (os.path.exists(base_path) and
+            not os.path.exists(backup_path) and
+            qtutils.version_check('5.15', compiled=False)):
+        os.mkdir(backup_path)
+        for filename in glob.glob(os.path.join(base_path, '*.yml')):
+            shutil.copy(filename, backup_path)
+
     try:
         os.mkdir(base_path)
     except FileExistsError:
@@ -594,3 +606,34 @@ def session_delete(name, *, force: bool = False) -> None:
                                     .format(e))
     else:
         log.sessions.debug("Deleted session {}.".format(name))
+
+
+def load_default(name):
+    """Load the default session.
+
+    Args:
+        name: The name of the session to load, or None to read state file.
+    """
+    if name is None and session_manager.exists('_autosave'):
+        name = '_autosave'
+    elif name is None:
+        try:
+            name = configfiles.state['general']['session']
+        except KeyError:
+            # No session given as argument and none in the session file ->
+            # start without loading a session
+            return
+
+    try:
+        session_manager.load(name)
+    except SessionNotFoundError:
+        message.error("Session {} not found!".format(name))
+    except SessionError as e:
+        message.error("Failed to load session {}: {}".format(name, e))
+    try:
+        del configfiles.state['general']['session']
+    except KeyError:
+        pass
+    # If this was a _restart session, delete it.
+    if name == '_restart':
+        session_manager.delete('_restart')
