@@ -90,7 +90,7 @@ def distribution() -> typing.Optional[DistributionInfo]:
         with open(filename, 'r', encoding='utf-8') as f:
             for line in f:
                 line = line.strip()
-                if (not line) or line.startswith('#'):
+                if (not line) or line.startswith('#') or '=' not in line:
                     continue
                 k, v = line.split("=", maxsplit=1)
                 info[k] = v.strip('"')
@@ -287,7 +287,7 @@ def _os_info() -> typing.Sequence[str]:
             versioninfo = ''
         else:
             versioninfo = '.'.join(info_tpl)
-        osver = ', '.join([e for e in [release, versioninfo, machine] if e])
+        osver = ', '.join(e for e in [release, versioninfo, machine] if e)
     elif utils.is_posix:
         osver = ' '.join(platform.uname())
     else:
@@ -343,7 +343,7 @@ def _chromium_version() -> str:
 
     Qt 5.9:  Chromium 56
     (LTS)    56.0.2924.122 (2017-01-25)
-             5.9.8: Security fixes up to 72.0.3626.121 (2019-03-01)
+             5.9.9: Security fixes up to 78.0.3904.108 (2019-11-18)
 
     Qt 5.10: Chromium 61
              61.0.3163.140 (2017-09-05)
@@ -355,8 +355,7 @@ def _chromium_version() -> str:
 
     Qt 5.12: Chromium 69
     (LTS)    69.0.3497.113 (2018-09-27)
-             5.12.6: Security fixes up to 76.0.3809.87 (2019-07-30)
-                     plus fix for CVE-2019-13720 from Chrome 78
+             5.12.8: Security fixes up to 80.0.3987.149 (2020-03-18)
 
     Qt 5.13: Chromium 73
              73.0.3683.105 (~2019-02-28)
@@ -364,7 +363,11 @@ def _chromium_version() -> str:
 
     Qt 5.14: Chromium 77
              77.0.3865.129 (~2019-10-10)
-             5.14.0: Security fixes up to 78.0.3904.108 (2019-11-18)
+             5.14.2: Security fixes up to 80.0.3987.132 (2020-03-03)
+
+    Qt 5.15: Chromium 80
+             80.0.3987.163 (2020-04-02)
+             5.15.0: Security fixes up to 81.0.4044.122 (2020-04-21)
 
     Also see https://www.chromium.org/developers/calendar
     and https://chromereleases.googleblog.com/
@@ -439,6 +442,8 @@ def version() -> str:
     if qapp:
         style = qapp.style()
         lines.append('Style: {}'.format(style.metaObject().className()))
+        platform_name = qapp.platformName()
+        lines.append('Platform plugin: {}'.format(platform_name))
 
     importpath = os.path.dirname(os.path.abspath(qutebrowser.__file__))
 
