@@ -52,9 +52,12 @@ def parse_args() -> argparse.Namespace:
                         default='auto',
                         help="PyQt version to install.")
     parser.add_argument('--pyqt-type',
-                        choices=['binary', 'source', 'link'],
+                        choices=['binary', 'source', 'link', 'wheels', 'skip'],
                         default='binary',
                         help="How to install PyQt/Qt.")
+    parser.add_argument('--pyqt-wheels-dir',
+                        default='wheels',
+                        help="Directory to get PyQt wheels from.")
     parser.add_argument('--virtualenv',
                         action='store_true',
                         help="Use virtualenv instead of venv.")
@@ -62,6 +65,12 @@ def parse_args() -> argparse.Namespace:
                         "asciidoc.py. If not given, it's searched in PATH.",
                         nargs=2, required=False,
                         metavar=('PYTHON', 'ASCIIDOC'))
+    parser.add_argument('--dev',
+                        action='store_true',
+                        help="Also install dev/test dependencies.")
+    parser.add_argument('--skip-docs',
+                        action='store_true',
+                        help="Skip doc generation.")
     parser.add_argument('--tox-error',
                         action='store_true',
                         help=argparse.SUPPRESS)
@@ -92,7 +101,7 @@ def run_venv(venv_dir: pathlib.Path, executable, *args: str) -> None:
         subprocess.run([str(venv_dir / subdir / executable)] +
                        [str(arg) for arg in args], check=True)
     except subprocess.CalledProcessError as e:
-        utils.print_col("Subprocess failed, exiting", 'red')
+        utils.print_error("Subprocess failed, exiting")
         sys.exit(e.returncode)
 
 
@@ -100,7 +109,7 @@ def pip_install(venv_dir: pathlib.Path, *args: str) -> None:
     """Run a pip install command inside the virtualenv."""
     arg_str = ' '.join(str(arg) for arg in args)
     utils.print_col('venv$ pip install {}'.format(arg_str), 'blue')
-    run_venv(venv_dir, 'python3', '-m', 'pip', 'install', *args)
+    run_venv(venv_dir, 'python', '-m', 'pip', 'install', *args)
 
 
 def show_tox_error(pyqt_type: str) -> None:
@@ -115,9 +124,9 @@ def show_tox_error(pyqt_type: str) -> None:
         raise AssertionError
 
     print()
-    utils.print_col('tox -e {} is deprecated. '
-                    'Please use "python3 scripts/mkvenv.py{}" instead.'
-                    .format(env, args), 'red')
+    utils.print_error('tox -e {} is deprecated. '
+                      'Please use "python3 scripts/mkvenv.py{}" instead.'
+                      .format(env, args))
     print()
 
 
@@ -134,9 +143,8 @@ def delete_old_venv(venv_dir: pathlib.Path) -> None:
     ]
 
     if not any(m.exists() for m in markers):
-        utils.print_col('{} does not look like a virtualenv, '
-                        'cowardly refusing to remove it.'.format(venv_dir),
-                        'red')
+        utils.print_error('{} does not look like a virtualenv, '
+                          'cowardly refusing to remove it.'.format(venv_dir))
         sys.exit(1)
 
     utils.print_col('$ rm -r {}'.format(venv_dir), 'blue')
@@ -151,24 +159,34 @@ def create_venv(venv_dir: pathlib.Path, use_virtualenv: bool = False) -> None:
             subprocess.run([sys.executable, '-m', 'virtualenv', venv_dir],
                            check=True)
         except subprocess.CalledProcessError as e:
-            utils.print_col("virtualenv failed, exiting", 'red')
+            utils.print_error("virtualenv failed, exiting")
             sys.exit(e.returncode)
     else:
         utils.print_col('$ python3 -m venv {}'.format(venv_dir), 'blue')
         venv.create(str(venv_dir), with_pip=True)
 
 
-def upgrade_pip(venv_dir: pathlib.Path) -> None:
-    """Upgrade pip inside a virtualenv."""
-    utils.print_title("Upgrading pip")
+def upgrade_seed_pkgs(venv_dir: pathlib.Path) -> None:
+    """Upgrade initial seed packages inside a virtualenv.
+
+    This also makes sure that wheel is installed, which causes pip to use its
+    wheel cache for rebuilds.
+    """
+    utils.print_title("Upgrading initial packages")
     pip_install(venv_dir, '-U', 'pip')
+    pip_install(venv_dir, '-U', 'setuptools', 'wheel')
 
 
-def pyqt_requirements_file(version: str):
-    """Get the filename of the requirements file for the given PyQt version."""
-    suffix = '' if version == 'auto' else '-{}'.format(version)
+def requirements_file(name: str) -> pathlib.Path:
+    """Get the filename of a requirements file."""
     return (REPO_ROOT / 'misc' / 'requirements' /
-            'requirements-pyqt{}.txt'.format(suffix))
+            'requirements-{}.txt'.format(name))
+
+
+def pyqt_requirements_file(version: str) -> pathlib.Path:
+    """Get the filename of the requirements file for the given PyQt version."""
+    name = 'pyqt' if version == 'auto' else 'pyqt-{}'.format(version)
+    return requirements_file(name)
 
 
 def install_pyqt_binary(venv_dir: pathlib.Path, version: str) -> None:
@@ -194,11 +212,27 @@ def install_pyqt_link(venv_dir: pathlib.Path) -> None:
     link_pyqt.link_pyqt(sys.executable, lib_path)
 
 
+def install_pyqt_wheels(venv_dir: pathlib.Path,
+                        wheels_dir: pathlib.Path) -> None:
+    """Install PyQt from the wheels/ directory."""
+    utils.print_title("Installing PyQt wheels")
+    wheels = [str(wheel) for wheel in wheels_dir.glob('*.whl')]
+    pip_install(venv_dir, *wheels)
+
+
 def install_requirements(venv_dir: pathlib.Path) -> None:
     """Install qutebrowser's requirement.txt."""
     utils.print_title("Installing other qutebrowser dependencies")
-    requirements_file = REPO_ROOT / 'requirements.txt'
-    pip_install(venv_dir, '-r', str(requirements_file))
+    requirements = REPO_ROOT / 'requirements.txt'
+    pip_install(venv_dir, '-r', str(requirements))
+
+
+def install_dev_requirements(venv_dir: pathlib.Path) -> None:
+    """Install development dependencies."""
+    utils.print_title("Installing dev dependencies")
+    pip_install(venv_dir,
+                '-r', str(requirements_file('dev')),
+                '-r', requirements_file('tests'))
 
 
 def install_qutebrowser(venv_dir: pathlib.Path) -> None:
@@ -219,21 +253,27 @@ def regenerate_docs(venv_dir: pathlib.Path,
 
     utils.print_col('venv$ python3 scripts/asciidoc2html.py {}'
                     .format(' '.join(a2h_args)), 'blue')
-    run_venv(venv_dir, 'python3', str(script_path), *a2h_args)
+    run_venv(venv_dir, 'python', str(script_path), *a2h_args)
 
 
 def main() -> None:
     """Install qutebrowser in a virtualenv.."""
     args = parse_args()
     venv_dir = pathlib.Path(args.venv_dir)
+    wheels_dir = pathlib.Path(args.pyqt_wheels_dir)
     utils.change_cwd()
 
     if args.tox_error:
         show_tox_error(args.pyqt_type)
         sys.exit(1)
-    elif args.pyqt_type == 'link' and args.pyqt_version != 'auto':
-        utils.print_col('The --pyqt-version option is not available when '
-                        'linking a system-wide install.', 'red')
+    elif (args.pyqt_version != 'auto' and
+          args.pyqt_type not in ['binary', 'source']):
+        utils.print_error('The --pyqt-version option is only available when '
+                          'installing PyQt from binary or source')
+        sys.exit(1)
+    elif args.pyqt_wheels_dir != 'wheels' and args.pyqt_type != 'wheels':
+        utils.print_error('The --pyqt-wheels-dir option is only available '
+                          'when installing PyQt from wheels')
         sys.exit(1)
 
     if not args.keep:
@@ -241,7 +281,7 @@ def main() -> None:
         delete_old_venv(venv_dir)
         create_venv(venv_dir, use_virtualenv=args.virtualenv)
 
-    upgrade_pip(venv_dir)
+    upgrade_seed_pkgs(venv_dir)
 
     if args.pyqt_type == 'binary':
         install_pyqt_binary(venv_dir, args.pyqt_version)
@@ -249,12 +289,20 @@ def main() -> None:
         install_pyqt_source(venv_dir, args.pyqt_version)
     elif args.pyqt_type == 'link':
         install_pyqt_link(venv_dir)
+    elif args.pyqt_type == 'wheels':
+        install_pyqt_wheels(venv_dir, wheels_dir)
+    elif args.pyqt_type == 'skip':
+        pass
     else:
         raise AssertionError
 
     install_requirements(venv_dir)
     install_qutebrowser(venv_dir)
-    regenerate_docs(venv_dir, args.asciidoc)
+    if args.dev:
+        install_dev_requirements(venv_dir)
+
+    if not args.skip_docs:
+        regenerate_docs(venv_dir, args.asciidoc)
 
 
 if __name__ == '__main__':
